#include "stdint.h"
#include "s3c44b0x.h"
#include "delay.h"
#include "bitbang.h"
#include "uart.h"
#include "nand.h"

void cpu_init()
{
   /* Set clock rate to 66.428MHz, just in case the JB has
      it set wrong
   */
   *PLLCON=0x55051;

   /* Be sure we're in cacheable memory
      Can the framebuffer be in cache too?
   */
   *NCACHBE0=0x80002000;
   *NCACHBE1=0x0;
}

void set_lcd_brightness(uint32_t brightness)
{
   /* Set LCD backlight brightness
      0 is dark, 0xa00 is full bright
   */
   *TCMPB1=brightness;
}

uint32_t compare_fail=0;

/* Compare a MMC sector to a memory buffer */
void mmc_compare(uint32_t sector, uint8_t* start)
{
   uint32_t i;

   mmc_byte(0xff);
   mmc_byte(0xff);
   mmc_byte(0xff);
   mmc_byte(0xff);

   /* read sector command */
   mmc_command(17, sector);

   /* spin until we get a zero reply, indicating command accepted */
   do
   {
      i=mmc_reply(1);
   } while (i!=0);


   /* spin until start of data token is received (0xfe for single read) */
   do
   {
      i=mmc_reply(1);
   } while (i!=0xfe);

   /* Read from the card */
   for (i=0; i<512; i++)
   {
      /* Total the bytes that diff */
      if (mmc_reply(1)!=(*start))
      {
         compare_fail++;
      }
      ++start;
   }

   /* give the MMC state machine 8 extra clocks */
   mmc_byte(0xff);
}

uint32_t temp_buffer[256];    /* 1k of ram */

/* Copy and readback to the MMC */
void copy_test()
{
   uint32_t i;
   compare_fail=0;

   for (i=0; i<0x2000; i++)
   {
      uart_tx_string("\r\nCopying sector 0x");
      uart_tx_string(itoh(i));

      nand_copy(i, temp_buffer);
      nand_reset();

      mmc_card_init();
      mmc_write_sector(i*0x200, (uint8_t*)temp_buffer);

      delay_ms(10);
      mmc_compare(i*0x200, (uint8_t*)temp_buffer);
   }
   uart_tx_string("\r\nNAND-MMC compare results: 0x");
   uart_tx_string(itoh(compare_fail));
   uart_tx_string(" bytes differ.");
}

int main(void)
{
   int32_t i;

   /* Init */
   cpu_init();
   uart_init();

   /* cool fade out to indicate demo running */
   for (i=0xa00; i>=0; i-=4)
   {
      set_lcd_brightness(i);
      delay_ms(1);
   }

   uart_tx_string("\r\n----------------------------------------------");
   uart_tx_string("\r\nFile: " __FILE__ ", (" __DATE__ ", " __TIME__);
   uart_tx_string(")\r\n");

   copy_test();
   mmc_card_demo();

   uart_tx_string("\r\n\r\nMMC test complete..");
   uart_tx_string("\r\n----------------------------------------------\r\n");

   /* Fade back in to indicate dump complete */
   for (i=0; i<=0xa00; i+=4)
   {
      set_lcd_brightness(i);
      delay_ms(1);
   }

   /* Spin forever */
   for (;;);

   return 0;
}
